/* ========================================
 * GT-521F32 Test Program
 *
 * ========================================
*/
#include "project.h"
#include <stdio.h>
#include <string.h>
#include "GT521F32_Defs.h"
#include "GT521F32.h"
#include "FPS_Utils.h"
#include "utils.h"
#include "main.h"

char str[STR_LENGTH+1] ; /* print buffer */
uint8_t template[TEMPLATE_SIZE] ;
char cmd[CMD_LENGTH+1] ;

cmd_func_type cmd_list[] = {
    { "help",    doHelp,        "help command" },
    { "led",     doLED,         "CMOS LED ON/OFF" },
    { "open",    doOpen,        "Open FPS" },
    { "close",   doClose,       "Close FPS" },
    { "baud",    doBaudRate,    "Set Baudrate" },
    { "count",   doEnrollCount, "Get Enrolled Count"},
    { "enroll",  doEnroll,      "Enroll a finger"},
    { "standby", doStandby,     "Enter Standby Mode"},
    { "wakeup",  doWakeUp,      "Wake up FPS"},
    { "erase",   doErase,       "Erase an ID"},
    { "eraseall", doEraseAll,   "Erase all IDs"},
    { "verify",   doVerify,     "Verify a finger print with ID"},
    { "identify", doIdentify,   "Identify a finger print from DB"},
    { 0, 0, 0 } 
} ;


void init_hardware(void)
{
    CyGlobalIntEnable; /* Enable global interrupts. */
    UART_Start() ; /* communication with the host PC */
    FPS_Clock_SetDivider(BAUD_COUNT_9600) ; /* 9600 baud */
    FPS_Start() ; /* communication with GT-521F32 (Finger Print Scanner) */
	VDAC8_3_3V_Start();	/* Start the Vref for the SIO pins. */
}

int main(void)
{
    FuncPtr func ;
    uint16_t adc_raw ;
    int16_t mV ;
    
    init_hardware() ;
    
    print("\x1b[2J\x1b[;H") ;
    sprintf(str, "GT-521F32 (Finger Print Scanner) Test (%s %s)\n", __DATE__, __TIME__ ) ;
    print(str) ;
    
    FPS_Open(1) ; /* Open FPS */

    for(;;)
    {
        print("> ") ;
        if (get_string(cmd)) {
            func = get_func(cmd) ;
            if (func) {
                func() ;
            } else {
                sprintf(str, "Unknown command: %s\n", cmd) ;
                print(str)  ;
            }
        }
    }
}

int doHelp(void)
{
    int i ;
    print("=== Command List ===\n") ;
    for (i = 0 ; cmd_list[i].cmd ; i++ ) {
        sprintf(str, "%10s : %s\n",
            cmd_list[i].cmd, cmd_list[i].description) ;
        print(str) ;
    }
    print("===\n") ;
    return( 0 ) ;
}

int doOpen(void) 
{
    int result ;
    result = FPS_Open(1) ;
    return( result ) ;
}

int doClose(void) 
{
    int result ;
    print("CLOSE Command : ") ;
    fps_change_baudrate(9600) ; /* set default baudrate */
    result = fps_close() ;
    if (result == CMD_ACK) {
        print("OK\n") ;
    } else if (result == CMD_NACK) {
        print("Nack Recieved\n") ;
    }    
    return( result ) ;
}

int doLED(void)
{
    char opt[16] ;
    int value = 0 ;
    if (get_string(opt)) {
        str2lower(opt) ;
        if (strcmp(opt, "on")==0) {
            value = 1 ;
        }
    }
    fps_led(value) ;
    return(value) ;
}

int doFastSearch(void)
{
    int result ;
    print("Fast Search : ") ;
    result = fps_first_search() ;
    if (result == CMD_ACK) {
        result = 0 ;
        print("OK\n") ;
    } else if (result == CMD_NACK) {
        print("Nack Recieved\n") ;
    }
    return( result ) ;
}

int doEnrollCount(void)
{
    int value ;
    print("Get Enroll Count : ") ;
    value = fps_get_enrolled_count() ;
    if (value < 0) {
        print("Nack Recieved\n") ;
    } else {
        sprintf(str, "Enrolled = %d\n", value) ;
        print(str) ;
    }
    return(value) ;
}

int doBaudRate(void) 
{
    int result ;
    int baud = 9600 ;
    char opt[32] ;
    if (get_string(opt)) {
        sscanf(opt, "%d", &baud) ;
    }
    result = FPS_Baudrate(baud) ;
    return( result ) ;
}

int doStandby(void)
{
    int result = 0 ;
    print("Enter Standby Mode : ") ;
    if (fps_enter_stadby() == 0) {
        print("OK\n") ;
    } else {
        print("Failed\n") ;
        result = 1 ;
    }
    return( result ) ;
}

int doWakeUp(void)
{
    int result = 0 ;
    print("Waking up the module : ") ;
    if (fps_wake_up() == 0) {
        print("OK\n") ;
    } else {
        print("Failed\n") ;
        result = 1 ;
    }
    return( result ) ;
}

int doErase(void)
{
    char opt[15] ;
    int result ;
    int id_to_erase = 0 ;
    if (get_string(opt)) {
        sscanf(opt, "%d", &id_to_erase) ;
    }
    sprintf(str, "Erasing ID[%d] ", id_to_erase) ;
    print(str) ;
    result = fps_delete_id(id_to_erase) ;
    if (result == CMD_ACK) {
        print("OK\n") ;
        result = 0 ;
    } else {
        print("Failed\n") ;
    }
    return( result ) ;
}

int doEraseAll(void)
{
    int result ;
    print("Erasing all IDs\n") ;
    result = fps_delete_all() ;
    if (result == CMD_ACK) {
        print("OK\n") ;
        result = 0 ;
    } else {
        print("Failed\n") ;
    }
    return( result ) ;
}

int doVerify(void) 
{
    int result ;
    char opt[16] ;
    int id_to_verify = 0 ; /* default test 0 */
    
    if (get_string(opt)) {
        sscanf(opt, "%d", &id_to_verify) ;
    }
    sprintf(str, "Touch the finger to verify with ID[%d] : ", id_to_verify) ;
    print(str) ;
    wait_finger_touch() ;
    print("touched\n") ;
    LED_ON ;
    CyDelay(100) ;
    
    print("Capturing the finger ") ;
    result = fps_capture_finger(1) ; /* best image */
    OK_OR_RETURN ;
    CyDelay(100) ;
    
    result = fps_verify(id_to_verify) ;
    CyDelay(100) ;
    LED_OFF ;
    if (result == 0) {
        print("Verified OK\n") ;
        result = 0 ;
    } else {
        print("Verify Failed\n") ;
    }
    return( result ) ;
}

int doIdentify(void) 
{
    int result ;
    print("Touch the finger to identify : ") ;
    wait_finger_touch() ;
    print("touched\n") ;
    LED_ON ;
    CyDelay(100) ;
    
    print("Capturing the finger ") ;
    result = fps_capture_finger(1) ; /* best image */
    OK_OR_RETURN ;
    CyDelay(100) ;
    
    result = fps_identify() ;
    CyDelay(100) ;
    LED_OFF ;
    if (result < MAX_ID) { /* within the range of IDs */
        sprintf(str, "Identified as ID[%d]\n", result) ;
    } else {
        sprintf(str, "Idenfify Failed\n" ) ;
    }
    print(str) ;
    return( result ) ;
}

int doEnroll(void) 
{
    int result = 0 ;
    int id ;
    id = fps_get_enrolled_count() ;
    CyDelay(100) ;
    result = FPS_EnrollAFingerAt(id) ;
    return( result ) ;
}

/* [] END OF FILE */
